@@ -1,4 +1,4 @@
-//  $Id: route-list.cc,v 4.9 2002/04/09 11:48:48 katie Exp $
+//  $Id: route-list.cc,v 4.10 2002/06/18 11:00:11 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -496,6 +496,11 @@ void RouteList::load(ASt _as) {
 
    const PrefixRanges *registered_routes = whois->expandAS(as);
 
+   if (!registered_routes) 	{
+	tcl_Eval("showWarning { No object for AS%d}", as);
+        return;
+   }
+	
    // Added by wlee
    if (registered_routes->isEmpty()) {
      tcl_Eval("showWarning { No route for AS%d}", as);
