@@ -1,4 +1,4 @@
-//  $Id: rptype.cc,v 4.9 2002/04/09 11:48:49 katie Exp $
+//  $Id: rptype.cc,v 4.9.4.1 2003/05/23 11:07:51 katie Exp $
 // Copyright (c) 2001,2002                        RIPE NCC
 //
 // All Rights Reserved
@@ -89,6 +89,12 @@ RPType *RPType::newRPType(char *name) {
       return new RPTypeIPv4Prefix;
    if (!strcasecmp(name, "address_prefix_range"))
       return new RPTypeIPv4PrefixRange;
+   if (!strcasecmp(name, "ipv6_address"))
+      return new RPTypeIPv6Address;
+   if (!strcasecmp(name, "ipv6_address_prefix"))
+      return new RPTypeIPv6Prefix;
+   if (!strcasecmp(name, "ipv6_address_prefix_range"))
+      return new RPTypeIPv6PrefixRange;
    if (!strcasecmp(name, "connection"))
       return new RPTypeConnection;
    if (!strcasecmp(name, "dns_name"))
@@ -411,6 +417,32 @@ Item *RPTypeIPv4PrefixRange::typeCast(const Item *item) const {
    return NULL;
 }
 
+////////////////////////////// RPTypeIPv6Address ////////////////////
+
+bool RPTypeIPv6Address::validate(const Item *item) const {
+   return (typeid(*item) == typeid(ItemIPV6));
+}
+
+////////////////////////////// RPTypeIPv6Prefix ////////////////////
+
+bool RPTypeIPv6Prefix::validate(const Item *item) const {
+   return (typeid(*item) == typeid(ItemPRFXV6));
+}
+
+////////////////////////////// RPTypeIPv6PrefixRange ////////////////////
+
+bool RPTypeIPv6PrefixRange::validate(const Item *item) const {
+   return (typeid(*item) == typeid(ItemPRFXV6Range));
+}
+
+Item *RPTypeIPv6PrefixRange::typeCast(const Item *item) const {
+   if (typeid(*item) == typeid(ItemPRFXV6))
+      return new ItemPRFXV6Range
+         (new IPv6PrefixRange(*((ItemPRFXV6 *) item)->prfxv6));
+   return NULL;
+}
+
+
 ////////////////////////////// RPTypeConnection ////////////////////
 
 bool RPTypeConnection::validate(const Item *item) const {
